/*
 * UNIX RFCNB (RFC1001/RFC1002) NetBIOS implementation
 * 
 * Version 1.0
 * RFCNB Defines
 * 
 * Copyright (C) Richard Sharpe 1996
 * 
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __SMBLIB_RFCNB_H
#define __SMBLIB_RFCNB_H

#include <stdio.h>        /* FILE */
#include <netinet/in.h>   /* in_addr */


/* if not compiling library
 */
#if !defined(SMB_IMPLEMENTATION) && !defined(UCHAR)
  #define UCHAR   unsigned char
  #define UWORD   unsigned short
  #define ULONG   unsigned long
  #define uint16  UWORD
#endif

#ifndef BOOL
#define BOOL  int
#endif

#ifndef TRUE
#define TRUE  1
#endif

#ifndef FALSE
#define FALSE 0
#endif


/* Defines we need
 */
#ifndef RFCNB_DEFAULT_PORT
#define RFCNB_DEFAULT_PORT 139
#endif

#ifndef RFCNB_MAX_STATS
#define RFCNB_MAX_STATS 1
#endif

/* Macros to extract things from the header. These are for portability
 * between architecture types where we are worried about byte order
 */
#define RFCNB_Pkt_Hdr_Len        4
#define RFCNB_Pkt_Sess_Len       72
#define RFCNB_Pkt_Retarg_Len     10
#define RFCNB_Pkt_Nack_Len       5
#define RFCNB_Pkt_Type_Offset    0
#define RFCNB_Pkt_Flags_Offset   1
#define RFCNB_Pkt_Len_Offset     2 /* Length is 2 bytes plus a flag bit */
#define RFCNB_Pkt_N1Len_Offset   4
#define RFCNB_Pkt_Called_Offset  5
#define RFCNB_Pkt_N2Len_Offset   38
#define RFCNB_Pkt_Calling_Offset 39
#define RFCNB_Pkt_Error_Offset   4
#define RFCNB_Pkt_IP_Offset      4
#define RFCNB_Pkt_Port_Offset    8

/* The next macro isolates the length of a packet, including the
 * bit in the flags
 */
#define RFCNB_Pkt_Len(p)  (PVAL(p, 3) | (PVAL(p, 2) << 8) |  \
                          ((PVAL(p, RFCNB_Pkt_Flags_Offset) & 1) << 16))

#define RFCNB_Pkt_Type(p) (CVAL(p, RFCNB_Pkt_Type_Offset))

#define RFCNB_Put_Pkt_Len(p, v) ((p)[1] = (((v) >> 16) & 1));   \
                                ((p)[2] = (((v) >> 8) & 0xFF)); \
                                ((p)[3] = ((v) & 0xFF));



/* Protocol defines we need
 */
#define RFCNB_SESSION_MESSAGE     0
#define RFCNB_SESSION_REQUEST     0x81
#define RFCNB_SESSION_ACK         0x82
#define RFCNB_SESSION_REJ         0x83
#define RFCNB_SESSION_RETARGET    0x84
#define RFCNB_SESSION_KEEP_ALIVE  0x85


/* Error responses
 *
 * these should follow the spec ... is there one ?
 */
#define RFCNBE_NoSpace        1   /* Could not allocate space for a struct */
#define RFCNBE_BadName        2   /* Could not translate a name            */
#define RFCNBE_BadRead        3   /* Read sys call failed                  */
#define RFCNBE_BadWrite       4   /* Write Sys call failed                 */
#define RFCNBE_ProtErr        5   /* Protocol Error                        */
#define RFCNBE_ConGone        6   /* Connection dropped                    */
#define RFCNBE_BadHandle      7   /* Handle passed was bad                 */
#define RFCNBE_BadSocket      8   /* Problems creating socket              */
#define RFCNBE_ConnectFailed  9   /* Connect failed                        */
#define RFCNBE_CallRejNLOCN  10   /* Call rejected, not listening on CN    */
#define RFCNBE_CallRejNLFCN  11   /* Call rejected, not listening for CN   */
#define RFCNBE_CallRejCNNP   12   /* Call rejected, called name not present */
#define RFCNBE_CallRejInfRes 13   /* Call rejetced, name ok, no resources   */
#define RFCNBE_CallRejUnSpec 14   /* Call rejected, unspecified error      */
#define RFCNBE_BadParam      15   /* Bad parameters passed ...             */
#define RFCNBE_Timeout       16   /* IO Timed out                          */


/*
 * Structures
 */

struct redirect_addr {
       struct in_addr        ip_addr;
       int                   port;
       struct redirect_addr *next;
     };

struct RFCNB_Con {
       int    fd;             /* File descripter for TCP/IP connection */
       int    errno;          /* last error code                       */
       int    timeout;        /* milli-secs before i/o times out       */
       int    redirects;      /* How many times we were redirected     */

       struct redirect_addr *redirect_list;  /* First is first address */
       struct redirect_addr *last_addr;
     };


#include <sys/packon.h>

struct RFCNB_Hdr {            /* RFC-NetBios header */
       UCHAR type;
       UCHAR flags;
       short len;
     };

#include <sys/packoff.h>


struct RFCNB_Pkt {
       char             *data;  /* The data in this portion */
       int               len;   /* length of data portion */
       struct RFCNB_Pkt *next;
     };

extern int RFCNB_errno;
extern int RFCNB_saved_errno;   /* Save this from point of error */

extern void (*Prot_Print_Routine)();

struct RFCNB_Pkt *RFCNB_Alloc_Pkt (int data_size);
void              RFCNB_Free_Pkt  (struct RFCNB_Pkt *pkt);

struct RFCNB_Con *RFCNB_Listen (unsigned short port);

struct RFCNB_Con *RFCNB_Call (const char *called_name,
                              const char *calling_name,
                              const char *called_addr,
                              unsigned short port);

int  RFCNB_Session_Req (struct RFCNB_Con *con,
                        const  char      *called_name,
                        const  char      *calling_name,
                        BOOL             *redirect,
                        struct in_addr   *dest_IP,
                        unsigned short   *port);

int  RFCNB_Send    (struct RFCNB_Con *, struct RFCNB_Pkt *, int len);
int  RFCNB_Recv    (struct RFCNB_Con *, struct RFCNB_Pkt *, int len);
int  RFCNB_Put_Pkt (struct RFCNB_Con *, struct RFCNB_Pkt *, int len);
int  RFCNB_Get_Pkt (struct RFCNB_Con *, struct RFCNB_Pkt *, int len);
int  RFCNB_Hangup  (struct RFCNB_Con *);

void RFCNB_Get_Error (char *buffer, int buf_len);

int  RFCNB_Get_Last_Error (void);

int  RFCNB_Get_Last_Errno (void);

int  RFCNB_Get_Error_Msg (int code, char *buf, int len);

void RFCNB_Register_Print_Routine (void (*fn)());

void RFCNB_CvtPad_Name (const char *name1, char *name2);

void RFCNB_AName_To_NBName (const char *aName, char *nbName);

void RFCNB_NBName_To_AName (const char *nbName, char *aName);

void RFCNB_Print_Hex (FILE*, const struct RFCNB_Pkt*, int ofs, int len);

void RFCNB_Print_Pkt (FILE*, const char*, const struct RFCNB_Pkt*, int len);

int  RFCNB_Name_To_IP (const char *, struct in_addr *);

int  RFCNB_IP_Connect (struct in_addr, int port);

int  RFCNB_Set_Sock_NoDelay (const struct RFCNB_Con *con, BOOL yes);

int  RFCNB_Set_Timeout (int seconds);

int  RFCNB_Close (int socket);

#endif  /* __SMBLIB_RFCNB_H */
